Package: shinystan
Title: Interactive Visual and Numerical Diagnostics and Posterior
        Analysis for Bayesian Models
Version: 2.7.0
Date: 2025-12-11
Authors@R: c(person("Jonah", "Gabry", role = c("aut", "cre"),
    email = "jgabry@gmail.com"),
      person("Duco", "Veen", role = "aut"),
      person(family = "Stan Development Team", role = "ctb"),
      person("Michael", "Andreae", role = "ctb"),
      person("Michael", "Betancourt", role = "ctb"),
      person("Bob", "Carpenter", role = "ctb"),
      person("Yuanjun", "Gao", role = "ctb"),
      person("Andrew", "Gelman", role = "ctb"),
      person("Ben", "Goodrich", role = "ctb"),
      person("Daniel", "Lee", role = "ctb"),
      person("Dongying", "Song", role = "ctb"),
      person("Rob", "Trangucci", role = "ctb"), 
      person("Visruth", "Srimath Kandali", role = "ctb"))
Maintainer: Jonah Gabry <jgabry@gmail.com>
Description: A graphical user interface for interactive Markov chain Monte
    Carlo (MCMC) diagnostics and plots and tables helpful for analyzing a
    posterior sample. The interface is powered by the 'Shiny' web
    application framework from 'RStudio' and works with the output of MCMC 
    programs written in any programming language (and has extended 
    functionality for 'Stan' models fit using the 'rstan' and 'rstanarm' 
    packages).
URL: https://mc-stan.org/shinystan/, https://discourse.mc-stan.org
BugReports: https://github.com/stan-dev/shinystan/issues/
License: GPL (>= 3)
Depends: R (>= 3.1.0), shiny (>= 1.0.3)
Imports: bayesplot (>= 1.5.0), colourpicker, DT (>= 0.2), dygraphs (>=
        1.1.1.2), ggplot2 (>= 3.4.0), gridExtra, gtools, markdown (>=
        0.7.4), methods, reshape2, rstan (>= 2.17.1), stats, shinyjs
        (>= 0.6.0), shinythemes (>= 1.0.1), threejs (>= 0.2.1), utils,
        xtable, xts (>= 0.9-7)
Suggests: cmdstanr (>= 0.9.0), coda, knitr (>= 1.9), posterior (>=
        1.0.0), rmarkdown (>= 0.8.1), rsconnect (>= 0.4.2), rstanarm
        (>= 2.17.4), testthat
Additional_repositories: https://stan-dev.r-universe.dev/
LazyData: true
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2025-12-11 18:24:13 UTC; jgabry
Author: Jonah Gabry [aut, cre],
  Duco Veen [aut],
  Stan Development Team [ctb],
  Michael Andreae [ctb],
  Michael Betancourt [ctb],
  Bob Carpenter [ctb],
  Yuanjun Gao [ctb],
  Andrew Gelman [ctb],
  Ben Goodrich [ctb],
  Daniel Lee [ctb],
  Dongying Song [ctb],
  Rob Trangucci [ctb],
  Visruth Srimath Kandali [ctb]
Repository: CRAN
Date/Publication: 2025-12-12 07:10:18 UTC
Built: R 4.6.0; ; 2026-04-25 12:41:30 UTC; unix
