# Generated by roxygen2: do not edit by hand

S3method(launch_shinystan,default)
S3method(launch_shinystan,shinystan)
export(as.shinystan)
export(deploy_shinystan)
export(drop_parameters)
export(generate_quantity)
export(is.shinystan)
export(launch_shinystan)
export(launch_shinystan_demo)
export(model_code)
export(model_name)
export(notes)
export(rename_model)
export(retrieve)
export(sso_info)
export(update_sso)
import(bayesplot)
import(dygraphs)
import(ggplot2)
import(methods)
import(shiny)
import(threejs)
import(utils)
import(xts)
importFrom(DT,datatable)
importFrom(colourpicker,colourInput)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(gtools,mixedsort)
importFrom(markdown,markdownToHTML)
importFrom(reshape2,melt)
importFrom(shinyjs,useShinyjs)
importFrom(shinythemes,shinytheme)
importFrom(stats,acf)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,time)
importFrom(stats,var)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)
