div(
  br(),
  helpText(
    style = "font-size: 11px;",
    "Use your mouse to highlight areas in the traceplot to zoom into. Double-click to reset.",
    "The number in the small black box in the bottom left corner controls the",
    em("roll period."),
    "If you specify a roll period of N the resulting graph will be a moving average,",
    "with each plotted point representing the average of N points in the data."
  )
)
